
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "gcr-certificate-chain.h" */


GType gcr_certificate_chain_status_get_type (void);
#define GCR_TYPE_CERTIFICATE_CHAIN_STATUS (gcr_certificate_chain_status_get_type())


GType gcr_certificate_chain_flags_get_type (void);
#define GCR_TYPE_CERTIFICATE_CHAIN_FLAGS (gcr_certificate_chain_flags_get_type())

/* enumerations from "gcr-certificate-request.h" */


GType gcr_certificate_request_format_get_type (void);
#define GCR_TYPE_CERTIFICATE_REQUEST_FORMAT (gcr_certificate_request_format_get_type())

/* enumerations from "gcr-column.h" */


GType gcr_column_flags_get_type (void);
#define GCR_TYPE_COLUMN_FLAGS (gcr_column_flags_get_type())

/* enumerations from "gcr-deprecated-base.h" */


GType gcr_importer_prompt_behavior_get_type (void);
#define GCR_TYPE_IMPORTER_PROMPT_BEHAVIOR (gcr_importer_prompt_behavior_get_type())

/* enumerations from "gcr-prompt.h" */


GType gcr_prompt_reply_get_type (void);
#define GCR_TYPE_PROMPT_REPLY (gcr_prompt_reply_get_type())

/* enumerations from "gcr-system-prompt.h" */


GType gcr_system_prompt_error_get_type (void);
#define GCR_TYPE_SYSTEM_PROMPT_ERROR (gcr_system_prompt_error_get_type())

/* enumerations from "gcr-system-prompter.h" */


GType gcr_system_prompter_mode_get_type (void);
#define GCR_TYPE_SYSTEM_PROMPTER_MODE (gcr_system_prompter_mode_get_type())

/* enumerations from "gcr-types.h" */


GType gcr_data_error_get_type (void);
#define GCR_TYPE_DATA_ERROR (gcr_data_error_get_type())


GType gcr_data_format_get_type (void);
#define GCR_TYPE_DATA_FORMAT (gcr_data_format_get_type())

G_END_DECLS

/* Generated data ends here */

