
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "gcr-collection-model.h" */


GType gcr_collection_model_mode_get_type (void);
#define GCR_TYPE_COLLECTION_MODEL_MODE (gcr_collection_model_mode_get_type())

G_END_DECLS

/* Generated data ends here */

